#!/usr/bin/python

# Python flecha: create an arrow from a vector
#
# Copyright 2017, Jacques Duflos <j.duflos@ymail.com>
# Please share and enjoy under the terms of the GPL v2 or later.

from gimpfu import *
from math import pi
from cmath import phase

def python_flecha(img, layer):
    # get the vectors and check if there is one
    vec = pdb.gimp_image_get_active_vectors(img)
    if vec == None:
        # afficher un message
        pdb.gimp_message_set_handler(0)
        pdb.gimp_message("No hay ninguna ruta seleccionada")
    else:
        pdb.gimp_undo_push_group_start(img)
        width = pdb.gimp_context_get_brush_size()

        # get the actual brush, and layer position
        old_brush = pdb.gimp_context_get_brush()
        position = pdb.gimp_image_get_item_position(img, layer)
        # create new layer to work on
        lay_arrow = pdb.gimp_layer_new(img, img.width, img.height, 1, "arrow", 100, 0)
        pdb.gimp_image_insert_layer(img, lay_arrow, None, position)
        pdb.gimp_image_set_active_layer(img, lay_arrow)

        # set the brush and draw the shaft
        pdb.gimp_context_set_brush("2. Hardness 100")
        pdb.gimp_edit_stroke_vectors(lay_arrow, vec)
        pdb.gimp_context_set_brush(old_brush)

        #draw the spike
        #get the final point
        type, num_points, controlpoints, closed = pdb.gimp_vectors_stroke_get_points(vec, 1)
        xf,yf = [controlpoints[index] for index in [num_points-4, num_points-3]]

        #get a point before the end and see the angle
        length = pdb.gimp_vectors_stroke_get_length(vec, 1, 1)
        ppf = pdb.gimp_vectors_stroke_get_point_at_dist(vec, 1, length-3, 1)
        xpf, ypf = ppf[0], ppf[1]
        angle = pi/2 + phase(complex(xf-xpf,yf-ypf))
        #create spike layer
        lay_spike = pdb.gimp_layer_new(img, width*2, width, 1, "spike", 100, 0)
        pdb.gimp_image_insert_layer(img, lay_spike, None, position)
        pdb.gimp_image_set_active_layer(img, lay_spike)
        #selecte the spike shape and fill it
        pdb.gimp_free_select(img, 6, [0,width,2*width,width,width,0], 2, True, False, 0)
        pdb.gimp_edit_bucket_fill(lay_spike, 0, 0, 100, 0, False, 0, 0)
        pdb.gimp_selection_none(img)
        # rotate, translate, merge layer and drop shadow
        pdb.gimp_item_transform_rotate(lay_spike, angle, False, width, width)
        pdb.gimp_layer_translate(lay_spike, xpf-width, ypf-width)
        new_layer = pdb.gimp_image_merge_down(img, lay_spike, 2)

        # create the shadow if not black
        color = gimp.get_foreground()
        colorcomp = [color[index] for index in [0,1,2]]
        if colorcomp != [0,0,0]:
            pdb.gimp_selection_layer_alpha(new_layer)
            pdb.gimp_selection_grow(img, 2)
            gimp.set_foreground((0,0,0))
            pdb.gimp_bucket_fill(new_layer, 0, 2, 100, 0, False, 0, 0)
            gimp.set_foreground(color)
        new_layer = pdb.gimp_image_merge_down(img, new_layer, 2)

        # reset the old brush and layer, display and close undo group
        pdb.gimp_image_set_active_layer(img, new_layer)
        pdb.gimp_selection_none(img)
        pdb.gimp_displays_flush()
        pdb.gimp_undo_push_group_end(img)

register(
	"python_fu_flecha",
	"Create an arrow from a vector",
	"Create an arrow from a single strok vector of specified width, color and border",
	"Jacques Duflos",
	"Jacques Duflos",
	"2017",
	"<Image>/Filters/Flecha",
	"*",
	[
	],
	[],
	python_flecha)


main()

