#!/usr/bin/python

# Python flecha: create an arrow from a vector
#
# Copyright 2017 CC0, Jacques Duflos <j.duflos@ymail.com>
# Please share and enjoy under the terms of the GPL v2 or later.

# History
# v1.0 2017-01-30 First published version
# v1.1 ?????????? make two scripts with and without options

# To do:
# if a tangant is specified at the last point, use it instead of calculating it
# find out how to put special caracters in the option window
# use the text_along_path script to add text along the arrow
# offer other spike shapes, and a starting form 

from gimpfu import *
from math import pi
from cmath import phase

def python_flecha_rapida(img, layer):
    width = pdb.gimp_context_get_brush_size()
    spike_ratio = 2
    color = gimp.get_foreground()
    is_border = [color[index] for index in [0,1,2]] != [0,0,0]
    pdb.gimp_undo_push_group_start(img)
    flecha(img, layer, None, width, spike_ratio, color, is_border, (0,0,0))
    pdb.gimp_undo_push_group_end(img)

def python_flecha_completa(img, layer, width, spike_ratio, color, is_border, border_color):
    pdb.gimp_undo_push_group_start(img)
    flecha(img, layer, None, width, spike_ratio, color, is_border, border_color)
    pdb.gimp_undo_push_group_end(img)

def flecha(img, layer, vec, width, spike_ratio, color, is_border, border_color):
    # if no vector specified, get active one and check
    if vec == None:
        vec = pdb.gimp_image_get_active_vectors(img)
    
    if vec == None:
        # afficher un message
        pdb.gimp_message_set_handler(0)
        pdb.gimp_message("No hay ninguna ruta seleccionada")
    else:
        # get the actual brush, and layer position
        old_color = pdb.gimp_context_get_foreground()
        old_brush = pdb.gimp_context_get_brush()
        old_size = pdb.gimp_context_get_brush_size()
        position = pdb.gimp_image_get_item_position(img, layer)
        # create new layer to work on
        lay_arrow = pdb.gimp_layer_new(img, img.width, img.height, 1, "arrow", 100, 0)
        pdb.gimp_image_insert_layer(img, lay_arrow, None, position)
        pdb.gimp_image_set_active_layer(img, lay_arrow)
        
        # set the brush and draw the shaft
        pdb.gimp_context_set_foreground(color)
        pdb.gimp_context_set_brush("2. Hardness 100")
        pdb.gimp_context_set_brush_size(width)
        pdb.gimp_edit_stroke_vectors(lay_arrow, vec)
        
        #draw the spike
        spike_width = width * spike_ratio
        #get the final point
        type, num_points, controlpoints, closed = pdb.gimp_vectors_stroke_get_points(vec, 1)
        xf,yf = [controlpoints[index] for index in [num_points-4, num_points-3]]
        
        #get a point before the end and see the angle
        length = pdb.gimp_vectors_stroke_get_length(vec, 1, 1)
        ppf = pdb.gimp_vectors_stroke_get_point_at_dist(vec, 1, length-3, 1)
        xpf, ypf = ppf[0], ppf[1]
        angle = pi/2 + phase(complex(xf-xpf,yf-ypf))
        #create spike layer
        lay_spike = pdb.gimp_layer_new(img, spike_width, spike_width * 0.5, 1, "spike", 100, 0)
        pdb.gimp_image_insert_layer(img, lay_spike, None, position)
        pdb.gimp_image_set_active_layer(img, lay_spike)
        #selecte the spike shape and fill it
        shape = [x * spike_width for x in [
            0.0 , 0.5,
            1.0 , 0.5,
            0.5 , 0.0]]
        pdb.gimp_free_select(img, 6, shape, 2, True, False, 0)
        pdb.gimp_edit_bucket_fill(lay_spike, 0, 0, 100, 0, False, 0, 0)
        pdb.gimp_selection_none(img)
        # rotate, translate, merge layer and drop shadow
        pdb.gimp_item_transform_rotate(lay_spike, angle, False, spike_width * 0.5, spike_width * 0.5)
        pdb.gimp_layer_translate(lay_spike, xf - spike_width *0.5, yf - spike_width * 0.5)
        new_layer = pdb.gimp_image_merge_down(img, lay_spike, 2)
        
        # create the shadow if asked
        if is_border:
            pdb.gimp_selection_layer_alpha(new_layer)
            pdb.gimp_selection_grow(img, 2)
            gimp.set_foreground(border_color)
            pdb.gimp_bucket_fill(new_layer, 0, 2, 100, 0, False, 0, 0)
        
        new_layer = pdb.gimp_image_merge_down(img, new_layer, 2)
        # reset the old brush and layer, display and close undo group
        pdb.gimp_context_set_foreground(old_color)
        pdb.gimp_context_set_brush(old_brush)
        pdb.gimp_context_set_brush_size(old_size)
        pdb.gimp_image_set_active_layer(img, new_layer)
        pdb.gimp_selection_none(img)
        pdb.gimp_displays_flush()


register(
    "python_fu_flecha_rapida",
    "Create an arrow from a vector",
    "Create an arrow from a single strok vector without options",
    "Jacques Duflos",
    "Jacques Duflos",
    "2017 CC0",
    "<Image>/Filters/Flecha rapida",
    "*",
    [
    ],
    [],
    python_flecha_rapida)

register(
    "python_fu_flecha_completa",
    "Create an arrow from a vector",
    "Create an arrow from a single strok vector with options",
    "Jacques Duflos",
    "Jacques Duflos",
    "2017 CC0",
    "<Image>/Filters/Flecha...",
    "*",
    [
    (PF_SPINNER, 'width', 'Ancho del eje:', 10,(1, 1000, 1)),
    (PF_SPINNER, 'spike_ratio', 'Tamano de la flecha x el ejen):', 2,(1, 100, 1)),
    (PF_COLOR, 'color', 'Color de la flecha:', (255,0,0)),
    (PF_BOOL , 'is_border', 'Con borde ?', True),
    (PF_COLOR, 'border_color', 'Color del borde:', (0,0,0))
    ],
    [],
    python_flecha_completa)

main()


